import 'bootstrap/dist/js/bootstrap';

const DEFAULT = {
    height: "400px",
    width: "600px",
    title: "对话框",
    content: "对话框内容",

    /*
        对话框body 内边距
     */
    padding: "15px",

    /*
        对话框头部颜色
     */
    headerColor: null,

    /*
        head 与 body 分割线颜色
     */
    splitColor: null,

    /*
        iframe 的 src 地址
     */
    frame: null,
    enableMax: false,
    onCloseEvent: null
};

// 修改对话框的大小
function setDialogSize($el, width, height) {
    // 修改modal-dialog这个div得宽高是为了居中问题
    $el.find(".modal-dialog").css("height", height);
    $el.find(".modal-dialog").css("width", width);

    // 这个div是真正显示的
    $el.find(".modal-content").css("height", height);
    $el.find(".modal-content").css("width", width);
}

class DialogWrapper {
    constructor(_el, _options) {
        this._options = Object.assign({}, JSON.parse(JSON.stringify(DEFAULT)), _options);
    }

    static getName() {
        return 'DialogWrapper';
    }

    static getScope() {
        return 'Global';
    }

    init() {

        let _this = this, height = _this._options.height, width = _this._options.width, content;
        // 如果宽高是百分比设置
        if(height && height.indexOf("%") > -1) {
            height = Number($(document.body).height()) * (Number(this._options.height.replace('%',''))/100) + "px";
        }
        if(width && width.indexOf("%") > -1) {
            width = Number($(document.body).width()) * (Number(this._options.width.replace('%',''))/100) + "px";
        }

        if(_this._options.frame) {
            content = "<iframe style='display: block' width='100%' height='100%' frameborder='0' src='" + _this._options.frame + "'></iframe>";
        }
        else {
            content = _this._options.content;
        }

        let headerColor = _this._options.headerColor ? "background-color: " + _this._options.headerColor : "",
            splitColor = _this._options.splitColor ? "border-top: 1px solid " + _this._options.splitColor : "";

        let $el = this._el = $(
            "<div class='modal fade cr-dialog-container' tabindex='-1' role='dialog'>" +
            "<div class='modal-dialog' role='document'>" +
            "<div class='modal-content'>" +
            "<div class='modal-header' style='" + headerColor + "'>" +
            "<button type='button' class='close' data-dismiss='modal' aria-label='Close'>" +
            "<span aria-hidden='true'>&times;</span></button>" +
            "<button type='button' class='close sizeTool maxTool'>" +
            "<span>&curren;</span></button>" +
            "<h4 class='modal-title'>" + _this._options.title + "</h4>" +
            "</div>" +
            "<div class='modal-body' style='padding:" + _this._options.padding + ";" + splitColor + "'>" + content +
            "</div></div></div></div>"
        ).appendTo(document.body);

        setDialogSize($el, width, height);

        // 放大,缩小
        $el.on('click', '.sizeTool', function() {
            let max = $(this).hasClass('minTool') ? false : true,
                winHeight = $(window).height(),
                winWidth = $(window).width();
            if(max === true) {
                setDialogSize($el, winWidth, winHeight);
                $el.find(".modal-dialog").css("margin", "0");
                $(this).removeClass("maxTool").addClass("minTool");
            }
            else {
                setDialogSize($el, width, height);
                $el.find(".modal-dialog").css(
                    "margin",
                    // ((winHeight-height.replace('px',''))/2) + "px " + ((winWidth-width.replace('px',''))/2) + "px"
                    "30px " + ((winWidth-width.replace('px',''))/2) + "px"
                );
                $(this).removeClass("minTool").addClass("maxTool");
            }
        });

        // 关闭事件
        $el.on('hidden.bs.modal', function () {
            _this._options.onCloseEvent && _this._options.onCloseEvent.call(_this);
        });

        return this;
    }

    open() {
        let $el = $(this._el);
        $el.modal('show');

        $el.on('hidden.bs.modal', function (e) {
            // 隐藏之后删除对话框
            $el.remove();
        })

        return this;
    }

    close() {
        this._el.modal('hide');
        return this;
    }

    defineCloseEvent(func) {
        this._options.onCloseEvent = func;
        return this;
    }
}

export default DialogWrapper