import '../../tip'
import DeepClone from '../../global/js/DeepClone'

const DEFAULT = {
    sourceUrl: null,

    /*
        是否叠加, 这个参数主要用于reload的时候
     */
    append: false,

    /*
        [{
            iconCls: 'glyphicon glyphicon-cloud', text: '菜单一', attributes: {url: 'www.a.com'}, children: [{
                iconCls: 'glyphicon glyphicon-cloud', text: '菜单1', attributes: {url: 'www.a.com'}
            }]
        }]
     */
    source: null,
    // 是否加载完成标志, 这个标志主要作用于openNode方法
    // loaded: false,

    /*
        默认选中下标
     */
    activeNode: 0,

    /*
        悬浮提示文字, 默认是提示text
     */
    tip: null,
    onLoadSuccess: function() {},
    onLoadComplete: function() {},

    /***
     *
     * @param $item 点击的叶子节点jquery对象
     * @param remote remote 后台传递过来的url
     * @param attrs attrs 后台传递过来的attributes
     */

    // 叶子节点点击事件
    onLeafClick: function($item, remote, attrs) {}
};


function renderMenu() {
    let options = this._options, _this = this;

    if(options.sourceUrl) {
        new Promise((resolve, reject) => {
            $.get(options.sourceUrl, function (data) {
                if (data.success) {
                    options.data = data.data;
                    renderMenuItem.call(_this);
                    !options.append && _this.openNode({index: options.activeNode});
                    options.onLoadSuccess.call(_this);
                }
                else {
                    $.TipWrapper({message: data.errorMsg}).open();
                }
                // options.loaded = true;
                options.onLoadComplete.call(_this);
            });
        });
    }
    else {
        options.data = options.source;
        renderMenuItem.call(this);
        !options.append && _this.openNode({index: options.activeNode});
    }
}

function renderMenuItem() {
    let _this = this, data = _this._options.data, $el = $(_this._el), html = "";

    data && data.map((item) => {

        let router = item.url,
            tip = item.tip ? item.tip : item.text,
            badge = item.badge ? `<span class="badge">${item.badge}</span>` : "",
            text = item.subText ? `
                <h5 class="list-group-item-heading">${item.text}</h5>
                <p class='list-group-item-text'>${item.subText}</p>
            ` : item.text,
            // 右侧添加打开或者关闭图标
            sign = item.children ? "<i class='cr-collapse-sign glyphicon glyphicon-folder-close'></i>" : "",
            attrs = item.attributes ? JSON.stringify(item.attributes) : "";

        html += "<li class='cr-collapse-item' data-attrs='" + attrs + "' key='" + router + "' title='" + tip + "'>";
        html += "<a href='#'><i class='" + item.iconCls + "'></i>  " + text + badge + sign + "</a>";

        // 如果存在子标签
        if(item.children) {
            // html += "<i class='cr-collapse-sign glyphicon glyphicon-folder-close'></i>";
            html += "<ul class='cr-collapse-inner'>";
            item.children.map((childItem) => {

                let router = childItem.url,
                    tip = childItem.tip ? childItem.tip : childItem.text,
                    badge = childItem.badge ? `<span class="badge">${childItem.badge}</span>` : "",
                    attrs = childItem.attributes ? JSON.stringify(childItem.attributes) : "",
                    text = childItem.subText ? `
                        <h5 class="list-group-item-heading">${childItem.text}</h5>
                        <p class='list-group-item-text'>${childItem.subText}</p>
                    ` : childItem.text;

                html += "<li title='" + tip + "' data-attrs='" + attrs + "' class='cr-collapse-item' key='" + router + "'><a href='#'>" + text + badge + "</a></li>";
            });
            html += "</ul>";
        }
        html += "</li>";

    });

    $el.append(html);
    $el.find(".cr-collapse-inner").hide();

    $el.on("click", ".cr-collapse-item", function(event) {
        let evt = window.event || event, $this = $(this),
            $a = $this.children("a"), $inner = $this.children(".cr-collapse-inner");

        if(evt.stopPropagation) {
            evt.stopPropagation();
        }
        else {
            evt.cancelBubble = true;
        }

        // 如果有内部菜单, 那就打开 or 关闭
        if($inner.length > 0) {
            $inner.slideToggle();
        }
        // 如果是叶子节点, 就触发click事件
        else {
            _this._options.onLeafClick.call(_this, $this, $this.attr("key"), $this.data("attrs"));
        }

        // 把其他选中的节点去除选中样式
        $el.find(".cr-collapse-item").not($this).children("a").removeClass("active");

        // 本节点选中 or 不选中
        $a.toggleClass("active");

        // li打开或者关闭图标切换
        $this.find('.cr-collapse-sign').toggleClass('glyphicon-folder-close').toggleClass('glyphicon-folder-open');

        return false;
    });
}

function innerOpenNode($el, options) {
    $el.children(".cr-collapse-item").map((index, item) => {
        // 若有内部元素,触发第一个内部菜单点击事件
        if($(item).find(".cr-collapse-inner").length > 0 && index === options.index) {
            $(item).trigger("click");
            $(item).find(".cr-collapse-inner .cr-collapse-item:eq(0)").trigger("click");
        }
        // 若没有内部元素,触发本元素
        else if($(item).find(".cr-collapse-inner").length === 0 && index === options.index) {
            $(item).trigger("click");
        }
    });
}

function innerGetNode($el, supIndex, subIndex) {
    let $supNode = $el.children(".cr-collapse-item:eq(" + supIndex + ")"),
        $inner = $supNode.find(".cr-collapse-inner");

    // 存在内部节点, 并且subIndex没有超出长度
    if($inner.length > 0 && $inner.children(".cr-collapse-item").length - 1 > subIndex) {
        let $node = $inner.children(".cr-collapse-item:eq(" + subIndex + ")");
        return {
            url: $node.attr("key"),
            text: $node.text(),
            attrs: $node.data("attrs")
        };
    }
    // 如果没有内部节点, 返回一级菜单的信息
    else if($inner.length === 0) {
        return {
            url: $supNode.attr("key"),
            text: $supNode.text(),
            attrs: $supNode.data("attrs")
        }
    }
    else {
        return null;
    }
}

class MenuWrapper {

    constructor(_el, options) {
        this._el = _el;
        this._options = Object.assign({}, DeepClone(DEFAULT), options);
    }

    static getName() {
        return "MenuWrapper";
    }

    init() {
        let $el = $(this._el), _this = this;
        $el.addClass("cr-collapse-outer");

        if($el.children(".cr-collapse-item").length === 0) {
            renderMenu.call(this);
        }

        return this;
    }
    openNode(options) {
        const $el = $(this._el);

        /*
            如果是远程获取数据, 这里有个延迟操作, loaded参数判定是否远程加载结束
            如果远程加载结束, 那么就不需要再覆盖onLoadSuccess方法来等待远程加载结束
            如果远程还没有加载结束, 那么就需要覆盖onLoadSuccess方法来等待远程加载结束之后再打开某个节点
         */

        // if(this._options.sourceUrl && !this._options.loaded) {
        //     this._options.onLoadSuccess = function() {
        //         innerOpenNode.call(null, $el, options);
        //     }
        // }
        // // 如果不是远程的,直接执行打开操作即可
        // else {
        //     innerOpenNode.call(null, $el, options);
        // }

        innerOpenNode.call(null, $el, options);


        return this;
    }
    getActiveNode() {
        const $el = $(this._el), $activeNode = $el.find(".cr-collapse-item a.active").closest(".cr-collapse-item");

        return {
            url: $activeNode.attr("key"),
            text: $activeNode.text(),
            attrs: $activeNode.data("attrs")
        }

    }
    getNode(supIndex, subIndex) {
        const $el = $(this._el);
        // 如果是远程获取数据,那么这里打开要等待loadsuccess之后才能执行,否则存在延迟不会打开
        // if(this._options.sourceUrl && !this._options.loaded) {
        //     this._options.onLoadSuccess = function() {
        //         return innerGetNode.call(null, $el, supIndex, subIndex);
        //     }
        // }
        // // 如果不是远程的,直接执行打开操作即可
        // else {
        //     return innerGetNode.call(null, $el, supIndex, subIndex);
        // }

        return innerGetNode.call(null, $el, supIndex, subIndex);
    }
    defineLeafClick(func) {
        this._options.onLeafClick = func;
        return this;
    }
    reload(options) {
        let $el = $(this._el);

        Object.assign(this._options, options);
        !options.append && $el.empty();
        renderMenu.call(this);
    }
    showEnd() {
        let $el = $(this._el);
        if($el.find(".cr-collapse-end").length === 0) {
            $el.append("<li class='cr-collapse-end'><a href='##'>----------暂无新的消息----------</a></li>");
        }
    }
}

export default MenuWrapper;