import DeepClone from '../../global/js/DeepClone';

const DEFAULT = {
    /*
        分页信息
     */
    pager: {
        pageSize: 10,
        curPage: 1,
        total: 0,
        totalPage: 0
    },

    /*
        是否单选
     */
    singleSelected: false,

    /*
        点击某行事件
     */
    clickRowEvent: function() {},

    /*
        选中某行事件
     */
    selectRowEvent: function() {}
};

function renderHead() {
    let _this = this, $el = _this._el, options = _this._options,
        columns = options.columns;

    $el.empty();

    let $thead = $("<thead><tr></tr></thead>").appendTo($el),
        $row = $thead.children("tr");

    columns && columns.forEach((column) => {

        if(column.field === "ck") {
            $row.append(`
                <td>选择</td>
            `);
        }
        else if(column.field === "orderNum") {
            $row.append(`
                <td>序号</td>
            `);
        }
        else if(!column.hidden) {
            $row.append(`
                <td>${column.title}</td>
            `);
        }

    });

    // 如果存在行按钮,添加操作一列
    if(options.rowTools && options.rowTools.length > 0) {
        $row.append("<td>操作</td>");
    }

}

function renderPage() {

    let _this = this, $el = _this._el, options = _this._options,
        total = options.pager.total || 0,
        pageSize = options.pager.pageSize, curPage = options.pager.curPage,
        totalPage = options.pager.totalPage = total % pageSize > 0 ? parseInt(total / pageSize) + 1 : total / pageSize;

    $el.closest(".row").next().remove();
    $el.closest(".row").next().remove();

    $el.closest(".row").after(`
        <br />
        <div class="row report-page-container">
            <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                <div class="btn-group btn-group-sm" role="group">
                    <button type="button" class="btn btn-default homePageBtn"><i class="glyphicon glyphicon-fast-backward"></i> 首页</button>
                    <button type="button" class="btn btn-default previousPageBtn">上一页</button>
                    <button type="button" class="btn btn-default nextPageBtn">下一页</button>
                    <button type="button" class="btn btn-default lastPageBtn"><i class="glyphicon glyphicon-fast-forward"></i> 末页</button>
                    <button type="button" class="btn btn-default refreshBtn"><i class="glyphicon glyphicon-refresh"></i> 刷新</button>
                </div>
            </div>
            <div class="col-xs-6 col-sm-6 col-md-6 col-lg-6">
                <div class="report-table-page pull-right">
                    每页显示<input class="pageSize" value="${pageSize}">条, 当前<input class="curPage" value="${curPage}">页, 
                    共<span class="total">${total}</span>条, <span class="totalPage">${totalPage}</span>页
                </div>
            </div>
        </div>
    `);

    let $pageContainer = $(".report-page-container");

    $pageContainer.find(".homePageBtn").on("click", toHomePage);
    $pageContainer.find(".previousPageBtn").on("click", toPreviousPage);
    $pageContainer.find(".nextPageBtn").on("click", toNextPage);
    $pageContainer.find(".lastPageBtn").on("click", toLastPage);
    $pageContainer.find(".refreshBtn").on("click", toRefresh);

}

function toHomePage() {
    let $table = $(this).closest('.report-page-container').prev().prev().find('.report-table'), _this = $table.data("self");

    _this._options.pager.curPage = 1;
    renderTable.call(_this);
    $(this).closest('.report-page-container').find('.curPage').val(_this._options.pager.curPage);
}

function toPreviousPage() {
    let $table = $(this).closest('.report-page-container').prev().prev().find('.report-table'), _this = $table.data("self"),
        curPage = _this._options.pager.curPage;

    if (curPage == 1) return false;
    _this._options.pager.curPage--;
    renderTable.call(_this);
    $(this).closest('.report-page-container').find('.curPage').val(_this._options.pager.curPage);
}

function toNextPage() {
    let $table = $(this).closest('.report-page-container').prev().prev().find('.report-table'), _this = $table.data("self"),
        curPage = _this._options.pager.curPage, totalPage = _this._options.pager.totalPage;

    if (curPage == totalPage) return false;
    _this._options.pager.curPage++;
    renderTable.call(_this);
    $(this).closest('.report-page-container').find('.curPage').val(_this._options.pager.curPage);
}

function toLastPage() {
    let $table = $(this).closest('.report-page-container').prev().prev().find('.report-table'), _this = $table.data("self"),
        totalPage = _this._options.pager.totalPage;
    _this._options.pager.curPage = totalPage;
    renderTable.call(_this);
    $(this).closest('.report-page-container').find('.curPage').val(_this._options.pager.curPage);
}

function toRefresh() {
    let $table = $(this).closest('.report-page-container').prev().prev().find('.report-table'), _this = $table.data("self"),
        curPage = $(this).closest('.report-page-container').find('.curPage').val(),
        pageSize = $(this).closest('.report-page-container').find('.pageSize').val(),
        totalPage = _this._options.pager.totalPage;

    if (!(/(^[1-9]\d*$)/.test(curPage))) {
        $.TipWrapper({message: "当前页输入非法字符"}).open();
        return false;
    }
    if (!(/(^[1-9]\d*$)/.test(pageSize))) {
        $.TipWrapper({message: "每页显示输入非法字符"}).open();
        return false;
    }
    if (curPage > totalPage) {
        curPage = totalPage;
        $(this).closest('.report-page-container').find('.curPage').val(curPage);
    }
    _this._options.pager.curPage = curPage;
    _this._options.pager.pageSize = pageSize;
    renderTable.call(_this);
}

function renderBody() {
    let _this = this, $el = _this._el, options = this._options,
        source = options.source, columns = options.columns,
        $tbody = $("<tbody></tbody>").appendTo($el),
        orderNum = 0;

    source && source.forEach((row) => {

        let $row = $("<tr></tr>").appendTo($tbody);
        orderNum++;

        columns && columns.forEach((column) => {

            if(column.field === "ck") {
                $row.append(`<td><i class='glyphicon glyphicon-unchecked'></i></td>`);
            }
            else if(column.field === "orderNum") {
                $row.append(`<td>${orderNum}</td>`);
            }
            else {
                let value = row[column.field];

                if(!column.hidden) {

                    if(value === "" || value === undefined || value === null) {
                        $row.append(`<td></td>`);
                    }
                    else {
                        if(column.formatter) {
                            value = column.formatter.call(null, value);
                        }
                        $row.append(`<td>${value}</td>`);
                    }

                }
            }

        });

        // 如果存在行按钮则添加
        if(options.rowTools && options.rowTools.length > 0) {

            let $td = $("<td></td>").appendTo($row);

            options.rowTools.forEach((rowTool, index) => {
                let color = rowTool.color ? rowTool.color.toLowerCase() : "primary";
                let $label = $(
                    "<a href='#' class='btn btn-xs btn-" + color + "'>" +
                    "<i class='" + rowTool.icon + "'></i> " + rowTool.text + "</a>"
                ).appendTo($td);

                index === rowTool.length || $td.append("&nbsp;");

                if(rowTool.handler) {
                    $label.click(function(evt) {
                        let event = evt || window.event;
                        rowTool.handler.call(null, event, row);
                    });
                }
            });
        }

        // 每条数据存入row的属性rowData中
        $row.data('rowData', row);
        // 绑定行点击事件
        $row.on('click', {_self: _this}, rowClk);

    });
}

function rowClk(event) {
    let $row = $(this), _self = event.data._self, $target = $(event.target);

    // 行按钮和列单击按钮点击,不触发行点击事件
    if($target.hasClass('cellClick') || $target.hasClass('btn-xs')) {
        return false;
    }

    // 点击之前是选中状态, 点击之后要取消选中状态
    if ($row.hasClass('cr-row-checked')) {

        // 多选才触发行点击事件
        if (!_self._options.singleSelected) {
            let index = $(this).closest("table").find("tbody tr").index($row);
            _self._options.clickRowEvent.call(null, event, index, $row.data('rowData'), false);
        }
        $row.find(".glyphicon-check").removeClass("glyphicon-check").addClass("glyphicon-unchecked");
    }
    // 点击之前不是选中状态, 点击之后要添加选中
    else {
        // 如果是单选去掉其他已经选中的行
        if (_self._options.singleSelected) {
            $row.siblings(".cr-row-checked").each((index, item) => {
                $(item).find(".glyphicon-check").removeClass("glyphicon-check").addClass("glyphicon-unchecked");
            }).removeClass('cr-row-checked');
        }

        let index = $(this).closest("table").find("tbody tr").index($row);

        _self._options.clickRowEvent.call(null, event, index, $row.data('rowData'), true);
        _self._options.selectRowEvent.call(null, event, index, $row.data('rowData'));

        $row.find(".glyphicon-unchecked").removeClass("glyphicon-unchecked").addClass("glyphicon-check");
    }

    $row.toggleClass("cr-row-checked");

}

function renderTable() {
    let _this = this, options = _this._options;

    this._loader = $.LoaderWrapper().open();

    if (this._options.sourceUrl) {

        let param = {
            curPage: options.pager.curPage,
            pageSize: options.pager.pageSize,
            searchFields: options.searchFields
        };

        // 将需要必须传递的表格参数,搜索参数,自定义参数进行合并一起传递到后台
        param = Object.assign(param, options.searchParam, options.remoteParam);

        $.post(options.sourceUrl, param, function(data) {
            if(data.success) {
                options.source = data.data.list;
                options.pager.total = data.data.total;
                renderHead.call(_this);
                renderBody.call(_this);
                renderPage.call(_this);
            }
            else {
                $.TipWrapper({message: data.errorMsg}).open();
            }
        });

    }
    else if (this._options.source) {
        let curPage = this._options.pager.curPage, pageSize = this._options.pager.pageSize,
            start = (curPage - 1) * pageSize,
            end = curPage * pageSize > this._options.source.length ? this._options.source.length : curPage * pageSize;

        this._options.pager.total = this._options.source.length;

        // 先深拷贝一份,存入data
        this._options.data = JSON.parse(JSON.stringify(this._options.source));

        /***
         * 如果存在过滤条件,先过滤,然后再分页,否则总页数和总条数是总数据的页数和条数
         *
         * 例如,数据[1,2,3],过滤掉3之后,总条数应该是2条
         * 如果先分页再过滤的画,总条数会显示成3条
         */

        // 如果存在搜索条件, 要进行过滤
        if (options.searchParam && options.searchFields.length > 0) {
            for (let i = this._options.data.length - 1; i >= 0; i--) {
                const row = this._options.data[i];
                options.searchFields.map((field, j) => {
                    // 当这一行数据的某个字段不存在搜索条件的时候,进行删除
                    if (row[field] && row[field].indexOf(options.searchParam.searchVal) < 0) {
                        this._options.data.splice(i, 1);
                    }
                });
            }
            this._options.pager.total = this._options.data.length;
        }

        // 如果开发者在使用的时候，传递了错误的数据给了table框架，就提示错误，并且停止渲染表格
        if(Object.prototype.toString.call(this._options.data) !== '[object Array]') {
            $.TipWrapper({message: "table框架接受的数据必须是个数组"}).open();
        }
        else {
            this._options.data = this._options.data.slice(start, end);

            renderRows.call(this);
        }

    }

}

class Table2Wrapper {
    constructor(_el, _options) {
        this._el = _el;
        this._options = Object.assign({}, DeepClone(DEFAULT), _options);
        // 默认设置当前页为第一页, 如果没有设置这个,外部不存在curPage属性的话,
        // Object.assign是直接拷贝外部不存在curPage属性的pager对象,导致curPage不存在
        this._options.pager.curPage || (this._options.pager.curPage = 1);
    }

    static getName() {
        return "Table2Wrapper";
    }

    init() {

        this._el.data("self", this);
        renderTable.call(this);

        return this;
    }

    reload(options) {

        // reload的时候可以对table的一些参数进行重新设置
        Object.assign(this._options, options);

        // 刷新的时候如果已经存在分页,直接出发分页按钮的点击事件即可
        if(this._pager) {
            this._pager.find(".refreshBtn").trigger('click');
        }
        // 刷新的时候如果还不存在分页,当没有数据的时候是没有分页的,
        // 添加完第一条数据要刷新的时候,分页还不存在,此时直接出发渲染表格方法
        else {
            renderTable.call(this);
        }
        return this;
    }
}

export default Table2Wrapper;